#!/usr/bin/env python
"""Batch script to generate 'release' commit."""

import argparse
import shutil
import subprocess
from datetime import date
from pathlib import Path
from textwrap import dedent

import tomllib
from jinja2 import Template

root = Path(__file__).parent.parent
parser = argparse.ArgumentParser()
parser.add_argument("level", type=str, choices=["major", "minor", "patch"])


def get_version() -> str:  # noqa: D103
    cmd = ["rye", "version"]
    proc = subprocess.run(cmd, stdout=subprocess.PIPE)
    return proc.stdout.decode("utf-8").strip()


def bump_version(level: str) -> str:  # noqa: D103
    """Bump version for sources."""
    cmd = ["rye", "version", "--bump", level]
    subprocess.run(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    return get_version()


def replace_version(target, current_version: str, new_version: str):
    """Change old/new version text from source."""
    src = root / target["filename"]
    lines = []
    from_ = target["search"].format(current_version=current_version)
    to_ = target["replace"].format(new_version=new_version)
    for line in src.read_text().split("\n"):
        if line == from_:
            lines.append(to_)
        else:
            lines.append(line)
    src.write_text("\n".join(lines))


def update_changes(current_version: str, new_version: str):
    """Generate changelog for new version.

    Currently, this works only generate template.
    """
    CHANGELOG_TEMPLATE = Template(
        dedent(
            """
        v{{version}}
        ={{'=' * version|length}}

        :date: {{now.strftime('%Y-%m-%d')}} (JST)

        Features
        --------

        Bug fixes
        ---------

        Miscellaneous
        -------------

        """
        ).strip()
    )
    now = date.today()
    target = root / "CHANGES.rst"
    move_to = root / "doc" / "changelogs" / f"v{current_version}.rst"
    move_to.parent.mkdir(parents=True, exist_ok=True)
    shutil.copy(target, move_to)
    target.write_text(CHANGELOG_TEMPLATE.render(version=new_version, now=now))


def main(args: argparse.Namespace):
    """Handle multi functions."""
    pyproject = tomllib.loads((root / "pyproject.toml").read_text())
    current_version = get_version()
    print(f"Current version: v{current_version}")
    new_version = bump_version(args.level)
    print(f"Next version:    v{new_version}")
    for target in pyproject["tool"]["local"]["bumpversion"]["files"]:
        replace_version(target, current_version, new_version)
    update_changes(current_version, new_version)


if __name__ == "__main__":
    args = parser.parse_args()
    main(args)



"""This is root of package."""

from bs4 import BeautifulSoup
from sphinx.application import Sphinx
from sphinx.config import Config
from sphinx.jinja2glue import BuiltinTemplateLoader

__version__ = "0.1.4"


class WithHtmxTemplateLoader(BuiltinTemplateLoader):  # noqa: D101
    def render(self, template: str, context: dict) -> str:  # noqa: D102
        out = super().render(template, context)
        if not template.endswith(".html"):
            return out
        soup = BeautifulSoup(out, "html.parser")
        soup.body["hx-boost"] = "true"
        for form in soup.find_all("form"):
            form["hx-boost"] = "false"
        return soup.prettify()


def setup_custom_loader(app: Sphinx, config: Config):
    """Inject extra values about htmx-boost into generated config."""
    # TODO: This extension must be loaded with html builders.
    config.template_bridge = "atsphinx.htmx_boost.WithHtmxTemplateLoader"
    if not hasattr(config, "html_js_files"):
        config.html_js_files = []
    # NOTE: Need control by config value?
    config.html_js_files.append("https://unpkg.com/htmx.org@1.9.10")


def setup(app: Sphinx):
    """Load as Sphinx-extension."""
    app.connect("config-inited", setup_custom_loader)
    return {
        "version": __version__,
        "env_version": 1,
        "paralell_read_safe": True,
    }



from atsphinx.htmx_boost import __version__

# -- Project information
project = "atsphinx-htmx-boost"
copyright = "2024, Kazuya Takei"
author = "Kazuya Takei"
release = __version__

# -- General configuration
extensions = ["atsphinx.htmx_boost", "sphinx.ext.githubpages"]
templates_path = ["_templates"]
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# -- Options for HTML output
html_theme = "furo"
html_title = f"{project} v{release}"
html_static_path = ["_static"]



"""Standard integration tests."""

import shutil
from io import StringIO
from pathlib import Path

import pytest
from bs4 import BeautifulSoup
from sphinx.testing.util import SphinxTestApp


@pytest.mark.sphinx("html")
def test__it(app: SphinxTestApp, status: StringIO, warning: StringIO):
    """Test to pass."""
    app.build()


@pytest.mark.parametrize(
    "theme", ["alabaster", "haiku", "nonav", "furo", "sphinx_rtd_theme"]
)
def test__work_on_theme(
    sphinx_test_tempdir: Path, rootdir: Path, make_app: callable, theme: str
):
    testroot = "root"
    srcdir = sphinx_test_tempdir / testroot
    if not srcdir.exists():
        testroot_path = rootdir / f"test-{testroot}"
        shutil.copytree(testroot_path, srcdir)

    app: SphinxTestApp = make_app(
        "html", srcdir=srcdir, confoverrides={"html_theme": theme}
    )
    app.build()
    soup = BeautifulSoup((app.outdir / "index.html").read_text(), "html.parser")
    assert "hx-boost" in soup.body.attrs


@pytest.mark.sphinx("html", confoverrides={"html_use_opensearch": True})
def test__with_opensearch(app: SphinxTestApp, status: StringIO, warning: StringIO):
    app.build()
